<?php

namespace App\DataFixtures;

use App\Entity\Group;
use App\Entity\User;
use Doctrine\Bundle\FixturesBundle\Fixture;
use Doctrine\Common\DataFixtures\DependentFixtureInterface;
use Doctrine\Persistence\ObjectManager;

class GroupFixtures extends Fixture implements DependentFixtureInterface
{
    public function load(ObjectManager $manager): void
    {
        $user_repository = $manager->getRepository(User::class);
        $group_author_1 = $user_repository->findOneBy(['email' => "defaultUser@localhost.com"]);
        if (! $group_author_1 instanceof User) {
            throw new \Exception("User does not exist.");
        }

        $group_author_2 = $user_repository->findOneBy(['email' => "defaultUser2@localhost.com"]);
        if (! $group_author_2 instanceof User) {
            throw new \Exception("User does not exist.");
        }

        $group_pop_1 = $this->createGroup($group_author_1, "Pop", $manager);
        $group_rock = $this->createGroup($group_author_1, "Rock", $manager);

        $group_pop_2 = $this->createGroup($group_author_2, "Pop", $manager);

        $author_1_capsules = $group_author_1->getCapsules();
        $author_1_capsules->toArray()[0]->addGroup($group_pop_1);

        $manager->flush();
    }

    private function createGroup(User $author, string $group_name, ObjectManager $manager): Group
    {
        $group = new Group();
        $group->setName($group_name);
        $group->setAuthor($author);
        $manager->persist($group);

        return $group;
    }

    public function getDependencies(): array
    {
        return [
            UserFixtures::class,
        ];
    }
}
